/* ***** BEGIN LICENSE BLOCK ***** 
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Jaap Haitsma.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * by the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Tracy Norris (tnorris@nc.rr.com), Jesper Kristensen (jesper@nc.rr.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var daysInMonth=new Array(31,28,31,30,31,30,31,31,30,31,30,31);

function doExit() {
	removeContentListeners();
	window.close();
	return true;
}

function doSave() {
	const nsIFilePicker = Components.interfaces.nsIFilePicker;

	var fp = Components.classes["@mozilla.org/filepicker;1"]
		.createInstance(nsIFilePicker);
	fp.init(window, "Dialog Title", nsIFilePicker.modeSave);
	fp.appendFilters(nsIFilePicker.filterAll | nsIFilePicker.filterText);

	var file;
	var rv = fp.show();
	if (rv == nsIFilePicker.returnOK){
		file = fp.file;
		file.create( Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 420 );
	}else if(rv == nsIFilePicker.returnReplace){
		file = fp.file;
		//TODO Should check to make sure file is writable
	}else{
		return;
	}

	var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
		.createInstance( Components.interfaces.nsIFileOutputStream );
	outputStream.init( file, 0x04 | 0x08 | 0x20, 420, 0 );
	var output = document.getElementById('testassist-event-text').value;
	var result = outputStream.write( output, output.length );
	outputStream.close();

}

function eventQueueOnLoad() {
	consoleDump('Opener: ' + opener);
	consoleDump('parent: ' + parent);
	// Attach to the browser to get select and load events
	opener.document.getElementById("content").addEventListener ("select", listenerSelect, true);
	opener.addEventListener('load', listenerLoad, true);
	
	attachContentListeners();	
}

function listenerLoad(event){
	consoleDump('Got load event' + event);
	var mainTabBox = opener.getBrowser().mTabBox;
	var myWin = opener.getBrowser().browsers[mainTabBox.selectedIndex].contentWindow;
	writeMessage('check_load:' + myWin.location.href);
	//removeContentListeners();
	attachContentListeners();	
}

function listenerSelect(event){
	consoleDump('Got select event' + event);
	//writeMessage('Select: ' + event.target.id);
	//removeContentListeners();
	attachContentListeners();
}

function removeContentListeners(){
	consoleDump("in removeContentListeners()");
	var browser = opener.document.getElementById("content");
	var oWindow = browser.browsers[browser.mTabBox.selectedIndex].contentWindow

	// Attach to the content window to get the
	oWindow.removeEventListener('keydown', listenerKeyDown, true);
	oWindow.removeEventListener('keyup', listenerKeyUp, true);
	oWindow.removeEventListener('keypress', listenerKeyPress, true);

	oWindow.removeEventListener('command', listenerCommand, true);
	oWindow.removeEventListener('focus', listenerFocus, true);

	oWindow.removeEventListener('mousedown', listenerMouseDown, true);
	oWindow.removeEventListener('mouseup', listenerMouseUp, true);
	oWindow.removeEventListener('click', listenerClick, true);

}

function attachContentListeners(){
	consoleDump("in attachContentListeners()");
	var browser = opener.document.getElementById("content");
	var oWindow = browser.browsers[browser.mTabBox.selectedIndex].contentWindow
	consoleDump("Content window: " + oWindow);

	// Attach to the content window to get the
	oWindow.addEventListener('keydown', listenerKeyDown, true);
	oWindow.addEventListener('keyup', listenerKeyUp, true);
	oWindow.addEventListener('keypress', listenerKeyPress, true);

	oWindow.addEventListener('command', listenerCommand, true);
	oWindow.addEventListener('focus', listenerFocus, true);

	oWindow.addEventListener('mousedown', listenerMouseDown, true);
	oWindow.addEventListener('mouseup', listenerMouseUp, true);
	oWindow.addEventListener('click', listenerClick, true);

}

function listenerClick(event){
	consoleDump('Got click event' + event);
	var message = 'click:';
	message += event.clientX +',';
	message += event.clientY +',';
	message += event.button;
	writeMessage(message);
}

function listenerMouseDown(event){
	consoleDump('Got mousedown event' + event);
	var message = 'mousedown:';
	message += event.clientX +',';
	message += event.clientY +',';
	message += event.button;
	writeMessage(message);
}

function listenerMouseUp(event){
	consoleDump('Got mouseup event' + event);
	var message = 'mouseup:';
	message += event.clientX +',';
	message += event.clientY +',';
	message += event.button;
	writeMessage(message);
}

function listenerKeyPress(event){
	consoleDump('Got keypress event' + event + ' with view of ' + event.view);
	var message = 'keypress:';
	message += event.ctrlKey +',';
	message += event.altKey +',';
	message += event.shiftKey +',';
	message += event.metaKey +',';
	message += event.keyCode +',';
	message += event.charCode;
	writeMessage(message);
}

function listenerKeyUp(event){
	consoleDump('Got keyup event' + event + ' with view of ' + event.view);
	var message = 'keyup:';
	message += event.ctrlKey +',';
	message += event.altKey +',';
	message += event.shiftKey +',';
	message += event.metaKey +',';
	message += event.keyCode +',';
	message += event.charCode;
	writeMessage(message);
}

function listenerKeyDown(event){
	consoleDump('Got keydown event' + event + ' with view of ' + event.view);
	var message = 'keydown:';
	message += event.ctrlKey +',';
	message += event.altKey +',';
	message += event.shiftKey +',';
	message += event.metaKey +',';
	message += event.keyCode +',';
	message += event.charCode;
	writeMessage(message);
}

function listenerFocus(event){
	consoleDump('Got focus event' + event);
	var oFocused = opener.document.commandDispatcher.focusedElement;
	if(oFocused){
//		if(oFocused.id.length > 0){
//			writeMessage('focus:' + oFocused.id);
//		}else{
			writeMessage('focus_xpath:' + getXPathQuery(oFocused));
//		}
	}
}

function listenerCommand(event){
	consoleDump('Got command event' + event);
	if(event.target && event.target.id.length > 0){
		writeMessage('command: ' + event.target.id);
	}
}

function writeMessage(message){
	var oTB = document.getElementById('testassist-event-text');
	if(oTB.value.length > 0){
		oTB.value = oTB.value + "\n" + message;
	}else{
		oTB.value = message;
	}
}

function getXPathQuery(targetNode){
	consoleDump('getXPathQuery('+targetNode+')');
	var query = '';
	var curNode = targetNode;
	while(curNode && curNode.nodeType != 9){ // Skip the document node
		consoleDump('looking at ' + curNode + ' of type ' + curNode.nodeType);
		nodeIndex = getTagIndex(curNode);
		if(nodeIndex == 0){
			query = '/' + curNode.tagName.toLowerCase() + query;
		}else{
			query = '/' + curNode.tagName.toLowerCase() +'[' + nodeIndex + ']' + query;
		}
		curNode = curNode.parentNode;
	}
	consoleDump('Query: ' + query);
	return query;
}

function getTagIndex(node){
	try{
	consoleDump('getTagIndex(' + node + ')');
	// Got to go up and down through the siblings to find the correct index
	var index = 0;
	var curNode = node.previousSibling;
	while(curNode){
		if(curNode.tagName == node.tagName){
			if(index == 0) index++;
			index++;
		}
		curNode = curNode.previousSibling;
	}

	if(index == 0){
		// We just need to see if we need the [1]
		curNode = node.nextSibling;
		while(curNode){
			if(curNode.tagName == node.tagName){
				index++;
				break;
			}
			curNode = curNode.nextSibling;
		}
	}
	consoleDump('Node Index: ' + index);
	return index;
	}catch(ex){
		alert(ex);
	}
}
