/* ***** BEGIN LICENSE BLOCK ***** 
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Jaap Haitsma.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * by the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Tracy Norris (tnorris@nc.rr.com), Jesper Kristensen (jesper@nc.rr.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
 // This holds the object for accessing the preferences
var testassistPrefs = Components.classes["@mozilla.org/preferences-service;1"].
                            getService(Components.interfaces.nsIPrefService).getBranch("testassist.");

var curElm = null;
var curTest = null;

setInternalPrefs();

// catch page loads
window.addEventListener('load', setButtonStatus, true);

// catch tab events
document.getElementById("content").addEventListener ("select", setButtonStatus, true); 

// preference observer
var prefChangeObserver = {
  observe :

  function(aSubject, aTopic, aData) {
	 setButtonStatus();
  }
};

// add observer to preference
//var qi = testassistPrefs.QueryInterface(Components.interfaces["nsIPrefBranchInternal"]);
//qi.addObserver("profile_list", prefChangeObserver, false);


// Set Button Status based on configuration
function setButtonStatus() {
	consoleDump("In setButtonStatus");

	return;

	// call doVisualize to turn the button on/off based on page content
	doVisualize();
	
	var testassistButton = document.getElementById("testassist-button");
	if (testassistPrefs.prefHasUserValue("profile_list")) {
		var profileList=testassistPrefs.getCharPref("profile_list");

		// if there are no profiles, disable the button (regardless of page content)
		if (profileList.length==0) {
			consoleDump("Disabling Attribute (no profiles)");
			testassistButton.setAttribute("disabled",true);
			testassistButton.removeAttribute("context");
		}
		// if there is only one profile, keep the status of the button, but make it not a menu button (and disable context)
		else if (profileList.indexOf(",")==-1) {
			consoleDump("UnMenuing Button");
			testassistButton.removeAttribute("type");
			testassistButton.removeAttribute("context");
		}
		// If there is more than one profile, leave the status of the button, but make it a menu button
		else {
			consoleDump("More than one profile, making menu-button");
			testassistButton.setAttribute("type","menu-button");
		}
	}
	else {
		consoleDump("Disabling Attribute (no config)");
		testassistButton.setAttribute("disabled",true);
		testassistButton.removeAttribute("type");
		testassistButton.removeAttribute("context");
	}
}

function populateTestAssistMenu(event) {
	consoleDump("In populateTestAssistMenu");
	
	clearMenu(event.target);
	
	for(i = 0; i < 10; i++){
		if(testassistPrefs.getCharPref('MRU_' + i) && testassistPrefs.getCharPref('MRU_' + i).length > 0){
			var menuitem = document.createElement("menuitem");
			menuitem.setAttribute("label",testassistPrefs.getCharPref('MRU_' + i)); // Add a space at the end to prevent theme conflicts (like profiles named "home")
			menuitem.setAttribute("name","testassist_mru_item");
			menuitem.setAttribute('mru_id', i);
			menuitem.setAttribute('oncommand', 'runMRUTest(event)');
			event.target.appendChild(menuitem);
		}
	}

	return true;
}

function runMRUTest(event){
	try{
		consoleDump('runMRUTest: ' + event.target);
		if(event.target.hasAttribute('mru_id')){
			var filename = testassistPrefs.getCharPref('MRU_' + event.target.getAttribute('mru_id'));
			curTest = new TestAssist(filename);
			curTest.startTest();
		}
	}catch(ex){
		alert('Error running MRU test: ' + ex);
	}
	return true;
}

function showOptions() {
	window.openDialog("chrome://testassist/content/testassist_pref_dialog.xul","testassist_pref_dialog","chrome,centerscreen");
}

function showEventQueue() {
	window.openDialog("chrome://testassist/content/testassist_event_dialog.xul","testassist_event_dialog","chrome,centerscreen");
}

function openTestFile(){
	//TODO Should add file to MRU list
	const nsIFilePicker = Components.interfaces.nsIFilePicker;

	var fp = Components.classes["@mozilla.org/filepicker;1"]
		.createInstance(nsIFilePicker);
	fp.init(window, "Dialog Title", nsIFilePicker.modeOpen);
	fp.appendFilters(nsIFilePicker.filterAll | nsIFilePicker.filterText);

	var rv = fp.show();
	if (rv == nsIFilePicker.returnOK){
		var file = fp.file;
		try{
			addTestToMRU(file.path);
			curTest = new TestAssist(file.path);
			curTest.startTest();
		}catch(ex){
			alert(ex);
		}
	}
}

function addTestToMRU(filename){
	for(i = 0; i < 10; i++){
		if(testassistPrefs.getCharPref('MRU_' + i) == filename){
			// Already in list
			return;
		}
	}
	
	// Move the items down one
	for(i = 9; i > 0; i--){
		testassistPrefs.setCharPref('MRU_' + i, testassistPrefs.getCharPref('MRU_' + (i - 1)));
	}
	testassistPrefs.setCharPref('MRU_0', filename);
}

function runTest(filename){
	try{
		var oFile = Components.classes["@mozilla.org/file/local;1"]
			.createInstance(Components.interfaces.nsILocalFile);
		oFile.initWithPath( filename );
		var is = Components.classes["@mozilla.org/network/file-input-stream;1"]
			.createInstance( Components.interfaces.nsIFileInputStream );
		is.init( oFile,0x01, 00004, null);
		is.QueryInterface(Components.interfaces.nsILineInputStream);
		var line = {};
		do {
			hasmore = is.readLine(line);
			//alert(line.value); 
			if(!runLine(line.value)){
				return false;
				is.close();
			}
		} while(hasmore);
		is.close();
	}catch(ex){
		alert(ex);
	}
}

function runLine(line){
	var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
		.getService(Components.interfaces.nsIPromptService);
	var command = line.substring(0, line.indexOf(':'));
	var param_string = line.substring(line.indexOf(':') + 1, line.length);
	var params = param_string.split(',');
	//alert("Command: " + command + "\nParam String: " + param_string + "\nParam Count: " + params.length);

	if(execCommand(command, params)){
		return true;
	}

	return promptService.confirm(window, "Continue Test?", "Click OK to continue running the test");
}

function execCommand(command, params){
	var browser = document.getElementById("content");
	var oWindow = browser.browsers[browser.mTabBox.selectedIndex].contentWindow
	consoleDump("Content window: " + oWindow);
	var box = oWindow.document.getBoxObjectFor(oWindow.document.documentElement);
	var screenDelta = { x: box.screenX, y: box.screenY };

	switch(command){
		case 'load':
			oWindow.status = "Loading URL " + params[0];
			oWindow.location.href = params[0];
			return true;
		
		case 'focus':
			curElm = oWindow.document.getElementById(params[0]);
			curElm.focus();
			return true;

		case 'focus_xpath':
			var elements = oWindow.document.evaluate(params[0], oWindow.document, null, XPathResult.ANY_TYPE,null);
			consoleDump(elements);
			curElm = elements.iterateNext();
			if(curElm && elements.iterateNext()){
				alert("Found too many elements for xpath:\n" + params[0]);
				return false;
			}else if(!curElm){
				alert("Could not find element for xpath:\n" + params[0]);
				return false;
			}
			curElm.focus();
			return true;

		case 'click':
			var x = params[0];
			var y = params[1];
			var evt = oWindow.document.createEvent("MouseEvents");
			evt.initMouseEvent('click', true, true, oWindow.document.defaultView, 
				0, x + screenDelta.x, y + screenDelta.y, x, y,
				false, false, false, false, params[1], null);
			if(curElm){
				curElm.dispatchEvent(evt);
			}else{
				oWindow.document.body.dispatchEvent(evt);
			}
			return true;

		case 'mouseup':
			var x = params[0];
			var y = params[1];
			var evt = oWindow.document.createEvent("MouseEvents");
			evt.initMouseEvent('mouseup', true, true, oWindow.document.defaultView, 
				0, x + screenDelta.x, y + screenDelta.y, x, y,
				false, false, false, false, params[1], null);
			if(curElm){
				curElm.dispatchEvent(evt);
			}else{
				oWindow.document.body.dispatchEvent(evt);
			}
			return true;

		case 'mousedown':
			var x = params[0];
			var y = params[1];
			var evt = oWindow.document.createEvent("MouseEvents");
			evt.initMouseEvent('mousedown', true, false, oWindow.document.defaultView, 
				0, x + screenDelta.x, y + screenDelta.y, x, y,
				false, false, false, false, params[1], null);
			if(curElm){
				curElm.dispatchEvent(evt);
			}else{
				oWindow.document.body.dispatchEvent(evt);
			}
			return true;

		case 'keyup':
			var evt = oWindow.document.createEvent("KeyEvents");
			evt.initKeyEvent("keyup", true, true, oWindow.document.defaultView,
				params[0] == 'true', params[1] == 'true', params[2] == 'true',
				params[3] == 'true', params[4], params[5]) 
			if(curElm){
				curElm.dispatchEvent(evt);
			}else{
				oWindow.document.body.dispatchEvent(evt);
			}
			return true;

		case 'keydown':
			var evt = oWindow.document.createEvent("KeyEvents");
			evt.initKeyEvent("keydown", true, true, oWindow.document.defaultView,
				params[0] == 'true', params[1] == 'true', params[2] == 'true',
				params[3] == 'true', params[4], params[5]) 
			if(curElm){
				curElm.dispatchEvent(evt);
			}else{
				oWindow.document.body.dispatchEvent(evt);
			}
			return true;

		case 'keypress':
			var evt = oWindow.document.createEvent("KeyEvents");
			evt.initKeyEvent("keypress", true, true, oWindow.document.defaultView,
				params[0] == 'true', params[1] == 'true', params[2] == 'true',
				params[3] == 'true', params[4], params[5]) 
			if(curElm){
				curElm.dispatchEvent(evt);
			}else{
				oWindow.document.body.dispatchEvent(evt);
			}
			return true;


		default:
			alert("Unknown command " + command);
			return false;
	}
}

function getElmAtPoint(x,y){
	consoleDump("Looking for element at point x:" + x + " y:" + y);
	var browser = document.getElementById("content");
	var oWindow = browser.browsers[browser.mTabBox.selectedIndex].contentWindow
	oWindow.document.addEventListener("mousemove", elmPointHandler, false);
	var event = oWindow.document.createEvent("MouseEvents");
	var box = oWindow.document.getBoxObjectFor(oWindow.document.documentElement);
	var screenDelta = { x: box.screenX, y: box.screenY };
	event.initMouseEvent("mousemove", true, false, oWindow.document.defaultView, 0,
		x + screenDelta.x, y + screenDelta.y, x, y,
		false, false, false, false, 0, null);
	oWindow.document.dispatchEvent(event);
	oWindow.document.removeEventListener("mousemove", elmPointHandler, false);
}

function elmPointHandler(event){
	curElm = event.explicitOriginalTarget;
	curElm.focus();
	consoleDump("Found element at point " + curElm);

	// reparent target if it is a text node to emulate IE's behavior
	if (curElm.nodeType == Node.TEXT_NODE)
		curElm = curElm.parentNode;

	// change an HTML target to a BODY target to emulate IE's behavior (if we are in an HTML document)

	event.preventDefault();
	event.stopPropagation();
}

function pause(numberMillis) {
	var now = new Date();
	var exitTime = now.getTime() + numberMillis;
	while (true) {
		now = new Date();
		if (now.getTime() > exitTime)
			return;
	}
}

function clearMenu(aParent) {
  var children = aParent.childNodes;
  for (var i = 0; i < children.length; i++) {
      if(children[i].hasAttribute("mru_id")){
        aParent.removeChild(children[i]);
			}
   }
}


function flattenText(node) {
	
	var flText = "";
	if (node.firstChild != null) {
		for (var c=0;c<node.childNodes.length && node.childNodes[c].nodeName.toLowerCase() != "input";c++) {
			var nodeindex = (node.childNodes.length - c) - 1; // do it backwards!
			flText = flText + flattenText(node.childNodes[c]);
		}
	}
	else {
		if (node.nodeValue != null) {
			flText = node.nodeValue;
		}
	}
	return flText;
}

function getLabelForNode(node) {	
	
	var prevNode = node;
	var nodeText = "";
	for(var backnodes=0;backnodes<5 && prevNode!=null;backnodes++) {
				
		if (prevNode.previousSibling != null) {
			prevNode = prevNode.previousSibling;
			nodeText = flattenText(prevNode) + nodeText;
		}
		else if (prevNode.parentNode != null) {
			prevNode = prevNode.parentNode;
			if (prevNode.nodeValue != null) {
				nodeText = prevNode.nodeValue;
			}
		}
		else {
			prevNode = null;
		}		
		
		if (prevNode.nodeName.toLowerCase() == 'input') {
			prevNode = null;
		}
	}		
	
	return nodeText;
}

function trim(s) {
	// Remove leading spaces and carriage returns
	while ((s.substring(0,1) == ' ') || (s.substring(0,1) == '\n') || (s.substring(0,1) == '\r')) {
		s = s.substring(1,s.length);
	}

	// Remove trailing spaces and carriage returns
	while ((s.substring(s.length-1,s.length) == ' ') || (s.substring(s.length-1,s.length) == '\n') || (s.substring(s.length-1,s.length) == '\r')) {
		s = s.substring(0,s.length-1);
	}
	
	return s;
}


function doVisualize() {
	
	if (testassistPrefs.prefHasUserValue("profile_list")) {			
		processFields(false);			
		if (testassistPrefs.getBoolPref("autohighlight") == true) {
			var linkElm = window.content.document.createElement("link");
			linkElm.setAttribute("id", "testassistVisualize");
			linkElm.setAttribute("href", "chrome://testassist/skin/visualize.css");
			linkElm.setAttribute("rel", "stylesheet");
			linkElm.setAttribute("type", "text/css");
			
			window.content.document.documentElement.childNodes[0].appendChild(linkElm);		
		}
	}
	else {
		document.getElementById("testassist-button").setAttribute("disabled","true");		
	}
}


function processFields(bChange) {

	var testassistPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("testassist.");
	var formFields = new Array("full_name","title","first_name","middle_name","last_name","suffix","phone_number","primary_line1",
				"primary_line2","primary_city","primary_state","primary_zip","primary_country","secondary_line1","secondary_line2",
				"card_name","card_type","card_number","card_verification","card_exp_day","card_exp_month","card_exp_year",
				"secondary_city","secondary_state","secondary_zip","secondary_country","primary_country_code","secondary_country_code","email_address");								
	var encryptedCCFields = "card_name,card_type,card_number,card_verification,card_exp_day,card_exp_month,card_exp_year";
	var profiles=testassistPrefs.getCharPref("profile_list");	
	var enableAutofill = false;	
	var password = "";
	var populateCCFields = false;
	
	if (profiles.length>0) {
		
		
		if (bChange) {
			var metaElements = window.content.document.getElementsByTagName("meta");
			for (var me = 0; me < metaElements.length; me++) {
				if (metaElements[me].name == "testassist" && metaElements[me].content == "encrypted") {							
					if (testassistPrefs.prefHasUserValue("password")) {
						if (checkPassword()) {
							populateCCFields = true;
							createAutofillKey();
						}
					}					
					me = metaElements.length;
				}
			}
		}
		
		var profileList=profiles.split(",");

		// select correct profile
		var profilenumber=0;
		for (var i=0;i<profileList.length;i++) {
			if (testassistPrefs.getCharPref("selected_profile_id") == profileList[i]) {
				profilenumber = i;
			}
		}

		var formValues = {};
		for (var i=1;i<formFields.length;i++) {
			formValues[formFields[i]]=testassistPrefs.getCharPref(profileList[profilenumber]+"."+formFields[i]);
		}
		
		// Copy primary address values to secondary if required
		if (testassistPrefs.getBoolPref(profileList[profilenumber]+".use_primary") == true) {
			formValues["secondary_city"] = formValues["primary_city"];
			formValues["secondary_country"] = formValues["primary_country"];
			formValues["secondary_country_code"] = formValues["primary_country_code"];
			formValues["secondary_line1"] = formValues["primary_line1"];
			formValues["secondary_line2"] = formValues["primary_line2"];
			formValues["secondary_state"] = formValues["primary_state"];
			formValues["secondary_zip"] = formValues["primary_zip"];
		}
		
		// Setup fullname
		formValues["full_name"] = formValues["first_name"];
		if (formValues["middle_name"] != "") {
			formValues["full_name"] = formValues["full_name"] + " " + formValues["middle_name"];
		}
		if (formValues["last_name"] != "") {
			formValues["full_name"] = formValues["full_name"] + " " + formValues["last_name"];
		}
		if (formValues["suffix"] != "") {
			formValues["full_name"] = formValues["full_name"] + ", " + formValues["suffix"];
		}
		//formFields[formFields.length] = "full_name";  //already included in the array above

		// get all form fields
		var formElements = new Array();		
		
		var inputElements = window.content.document.getElementsByTagName("input");
		for (var j = 0; j < inputElements.length; j++) {
			formElements[formElements.length] = inputElements[j];
		}
		var selectElements = window.content.document.getElementsByTagName("select");
		for (var j = 0; j < selectElements.length; j++) {
			formElements[formElements.length] = selectElements[j];
		}
		

		var encryptedField = false;

		// loop through all input fields
		for (var j = 0; j < formElements.length; j++)	{
			// get attributes
			var elmType = formElements[j].type.toLowerCase();
			var elmName = formElements[j].name.toLowerCase();
			             			
			if (elmType == "input" || elmType == "text" || elmType == "textarea" || elmType == "password" || elmType == "select-one" || elmType == "select-multiple") {	
												
				var match = false;
				for (var l=0;l<formFields.length;l++) {					
					var fieldnamelist = testassistPrefs.getCharPref("form_name." + formFields[l]);
					var fieldnames = fieldnamelist.split(",");	
					if (fieldnames.length > 0) {
						for (var k=0;k<fieldnames.length;k++) {
							if (elmName == fieldnames[k].toLowerCase()) {
								match = true;								
								if (bChange) {
									if (encryptedCCFields.indexOf(formFields[l]) != -1) {
										if (populateCCFields) {			
											var decrypted = byteArrayToString(rijndaelDecrypt(hexToByteArray(formValues[formFields[l]]),testassistKey,"CBC"));											
											
											if (elmType == "select-one" || elmType == "select-multiple") {
												var selElement = formElements[j];
												for (var c=0; c<selElement.length;c++ ) {
													if (selElement[c].value.toLowerCase() == decrypted.toLowerCase()) {
														selElement[c].selected = true;
													}
												}			                        

											}
											else {
												formElements[j].value =	decrypted;										
											}											
										}
									}	
									else {
										if (elmType == "select-one" || elmType == "select-multiple") {
											var selElement = formElements[j];
											for (var c=0; c<selElement.length;c++ ) {
												if (selElement[c].value.toLowerCase() == formValues[formFields[l]].toLowerCase()) {
													selElement[c].selected = true;
												}
											}			                        
										
										}
										else {
											formElements[j].value =	formValues[formFields[l]];										
										}
									}
															
								}								
								else {
									formElements[j].setAttribute("testassistsupport","true");
									enableAutofill = true;
									if (encryptedCCFields.indexOf(formFields[l]) != -1) {
										encryptedField = true;
									}								
								}
							}
						}
					}			
				}	
				if (!match) {  // try field substring matching
					for (var l=0;l<formFields.length;l++) {					
						var fieldnamelist = testassistPrefs.getCharPref("form_label_name." + formFields[l]);
						var fieldnames = fieldnamelist.split(",");	
						if (fieldnames.length > 0) {
							for (var k=0;k<fieldnames.length;k++) {
								if (elmName.indexOf(fieldnames[k].toLowerCase()) != -1) {
									match = true;
									if (bChange) {
										if (encryptedCCFields.indexOf(formFields[l]) != -1) {
											if (populateCCFields) {			
												var decrypted = byteArrayToString(rijndaelDecrypt(hexToByteArray(formValues[formFields[l]]),testassistKey,"CBC"));											

												if (elmType == "select-one" || elmType == "select-multiple") {
													var selElement = formElements[j];
													for (var c=0; c<selElement.length;c++ ) {
														if (selElement[c].value.toLowerCase() == decrypted.toLowerCase()) {
															selElement[c].selected = true;
														}
													}			                        

												}
												else {
													formElements[j].value =	decrypted;										
												}											
											}
										}	
										else {
											if (elmType == "select-one" || elmType == "select-multiple") {
												var selElement = formElements[j];
												for (var c=0; c<selElement.length;c++ ) {
													if (selElement[c].value.toLowerCase() == formValues[formFields[l]].toLowerCase()) {
														selElement[c].selected = true;
													}
												}			                        

											}
											else {
												formElements[j].value =	formValues[formFields[l]];										
											}
										}

									}								
									else {
										formElements[j].setAttribute("testassistsupport","true");
										enableAutofill = true;
										if (encryptedCCFields.indexOf(formFields[l]) != -1) {
											encryptedField = true;
										}								
									}
								}
							}
						}			
					}
				}
				if (!match) {  // try label matching
					consoleDump("No field name match for: " + elmName);
					var labelText = getLabelForNode(formElements[j]).toLowerCase();
					if (labelText.length > 50) {
						labelText = labelText.substring(labelText.length-50,labelText.length);
					}
					consoleDump("Trying with following label: " + labelText);						

					for (var l=0;l<formFields.length;l++) {					
						var fieldnamelist = testassistPrefs.getCharPref("form_label_name." + formFields[l]);
						var fieldnames = fieldnamelist.split(",");	
						if (fieldnames.length > 0) {
							for (var k=0;k<fieldnames.length;k++) {
								if (labelText.indexOf(fieldnames[k].toLowerCase()) != -1) {
									match = true;
									if (bChange) {
										if (encryptedCCFields.indexOf(formFields[l]) != -1) {
											if (populateCCFields) {			
												var decrypted = byteArrayToString(rijndaelDecrypt(hexToByteArray(formValues[formFields[l]]),testassistKey,"CBC"));											

												if (elmType == "select-one" || elmType == "select-multiple") {
													var selElement = formElements[j];
													for (var c=0; c<selElement.length;c++ ) {
														if (selElement[c].value.toLowerCase() == decrypted.toLowerCase()) {
															selElement[c].selected = true;
														}
													}			                        

												}
												else {
													formElements[j].value =	decrypted;										
												}											
											}
										}	
										else {
											if (elmType == "select-one" || elmType == "select-multiple") {
												var selElement = formElements[j];
												for (var c=0; c<selElement.length;c++ ) {
													if (selElement[c].value.toLowerCase() == formValues[formFields[l]].toLowerCase()) {
														selElement[c].selected = true;
													}
												}			                        

											}
											else {
												formElements[j].value =	formValues[formFields[l]];										
											}
										}

									}								
									else {
										formElements[j].setAttribute("testassistsupport","true");
										enableAutofill = true;
										if (encryptedCCFields.indexOf(formFields[l]) != -1) {
											encryptedField = true;
										}								
									}
								}
							}
						}			
					}
	
				}																	
			}
		}
		if (encryptedField ) {
			var metaElm = window.content.document.createElement("meta");
			metaElm.setAttribute("name", "testassist");
			metaElm.setAttribute("content", "encrypted");			
			window.content.document.documentElement.childNodes[0].appendChild(metaElm);		
		}
	}
	
	if (!bChange) {
		if (enableAutofill) {
			document.getElementById('testassist-button').removeAttribute("disabled");			
			document.getElementById('testassist-button').setAttribute("context","testassist-menu");			
		}
		else {
			document.getElementById('testassist-button').setAttribute("disabled","true");	
			document.getElementById('testassist-button').removeAttribute("context");	
		}
	}
}


